require 'spec_helper'

RSpec.describe NsSettingsUi::SettingsController, type: :controller do
  routes { NsSettingsUi::Engine.routes }

  def sample_setting(value: 0, default_value: 0)
    key = Array('aa'..'zz').sample(2).join('.')
    NsSettingsUi::Setting.create(key: key, value: value, default_value: default_value)
  end

  describe 'GET index' do
    let!(:settings) { Array.new(8) { sample_setting }.sort_by(&:key) }
    subject { get :index }

    it 'assigns all settings' do
      subject
      expect(assigns(:settings)).to eq(settings)
    end

    context 'with views' do
      render_views
      it { is_expected.to be_successful }
    end
  end

  shared_examples_for 'sets setting and renders update' do
    it 'assigns the setting' do
      expect(assigns(:setting)).to eq(setting)
    end

    it 'responds with 200' do
      expect(response).to be_successful
    end

    it 'sets flash message' do
      is_expected.to render_template('ns_settings_ui/settings/_update')
    end
  end

  describe 'PUT reset' do
    let(:setting) { sample_setting(value: 'a', default_value: 'b') }
    let(:params) { {id: setting.id, format: :js} }

    before { get :reset, params: params }

    it 'resets setting value to the default value' do
      expect(assigns(:setting).value).to eq(setting.default_value)
    end

    it_behaves_like 'sets setting and renders update'
  end

  describe 'PUT update' do
    let(:setting) { sample_setting }
    let(:setting_params) { {value: '5', default_value: '6', key: 'matching.cluster_size'} }
    let(:params) { {id: setting.id, setting: setting_params, format: :js} }

    before { put :update, params: params }

    it 'updates value' do
      expect(assigns(:setting).value).to eq(setting_params[:value])
    end

    it 'updates default value' do
      expect(assigns(:setting).default_value).to eq(setting_params[:default_value])
    end

    it 'does not update key' do
      expect(assigns(:setting).key).to eq(setting.key)
    end

    it_behaves_like 'sets setting and renders update'
  end
end
